@@ -5,9 +5,9 @@ import pytest
 
 def _check(return_val):
     """Checks the return code of the C calls"""
-    if isinstance(return_val, int) and return_val == 1:
+    if return_val == 1 and isinstance(return_val, int):
       return
-    if isinstance(return_val, bool) and return_val == True:
+    if return_val == True and isinstance(return_val, bool):
       return
 
     raise Exception("Cipher exception: Unknown type %s or value %s" % (str(type(return_val)), str(return_val)))
@@ -73,7 +73,7 @@ class Cipher(object):
         """Return the OpenSSL nid of the cipher and mode."""
         return int(self.alg.nid)
 
-    def op(self, key, iv, enc=1):
+    def op(self, key, iv=None, enc=1):
         """Initializes a cipher operation, either encrypt or decrypt 
         and returns a CipherOperation object
 
@@ -83,12 +83,17 @@ class Cipher(object):
             enc (int): set to 1 to perform encryption, or 0 to perform decryption.
 
         """
+
+        if iv is None:
+            iv = _FFI.NULL
+
         c_op = CipherOperation(enc)
         _check( len(key) == self.len_key())
         _check( enc in [0,1] )
        
         if not self.gcm:
-            _check( len(iv) == self.len_IV())
+            if iv != _FFI.NULL:
+                _check( len(iv) == self.len_IV())
             _check( _C.EVP_CipherInit_ex(c_op.ctx, 
                 self.alg,  _FFI.NULL, key, iv, enc) )
 
@@ -223,6 +228,34 @@ class CipherOperation(object):
         _C.EVP_CIPHER_CTX_init(self.ctx)
         self.cipher = None
         self.xenc = xenc
+
+    def set_padding(self, pad):
+        """Sets the padding on or off, accodring to pad (bool).
+
+        Example:
+            >>> from os import urandom
+            >>> aes = Cipher("AES-128-ECB")     # Init AES in Electronic codebook mode
+            >>> key = urandom(16)
+            >>> iv  = None
+            >>>
+            >>> # Get a CipherOperation object for encryption
+            >>> enc = aes.enc(key, iv)
+            >>> enc.set_padding(False)
+            >>> ref = b"A" * 16
+            >>> ciphertext = enc.update(ref)
+            >>> ciphertext += enc.finalize()
+            >>> len(ciphertext)
+            16
+            >>> # Get a CipherOperation object for decryption
+            >>> dec = aes.dec(key, iv)
+            >>> dec.set_padding(False)
+            >>> plaintext = dec.update(ciphertext)
+            >>> plaintext += dec.finalize()
+            >>> plaintext == ref # Check resulting plaintest matches referece one.
+            True
+
+        """
+        _check( _C.EVP_CIPHER_CTX_set_padding(self.ctx, pad) )
             
     def update(self, data):
         """Processes some data, and returns a partial result."""
@@ -566,4 +599,21 @@ def test_quick_assoc():
     p = aes.quick_gcm_dec(b"A"*16, b"A"*16, c, t, assoc=b"blah")
     assert p == b"Hello"
 
+def test_ecb():
+        key = b"\x02" * 16
+        data = b"\x01" * 16 
+
+        assert len(data) == 16
+        aes = Cipher("AES-128-ECB")
+        enc = aes.enc(key, None)
+        c = enc.update(data)
+        c += enc.finalize()
+        
+        assert len(data) == 16
+        aes = Cipher("AES-128-ECB")
+        enc = aes.dec(key, None)
+        c1 = enc.update(c)
+        c1 += enc.finalize()
+        assert c1 == data
+
 # pylint: enable=unused-variable,redefined-outer-name
